@@ -1,17 +1,19 @@
 import os
-import numpy as np
-from moviepy.audio.io.ffmpeg_audiowriter import ffmpeg_audiowrite
-from moviepy.decorators import requires_duration
-from moviepy.tools import (deprecated_version_of,
-                           extensions_dict)
 
-from moviepy.Clip import Clip
+import numpy as np
+import proglog
 from tqdm import tqdm
 
+from moviepy.audio.io.ffmpeg_audiowriter import ffmpeg_audiowrite
+from moviepy.Clip import Clip
+from moviepy.decorators import requires_duration
+from moviepy.tools import deprecated_version_of, extensions_dict
+
+
 class AudioClip(Clip):
     """ Base class for audio clips.
     
-    See ``SoundClip`` and ``CompositeSoundClip`` for usable classes.
+    See ``AudioFileClip`` and ``CompositeSoundClip`` for usable classes.
     
     An AudioClip is a Clip with a ``make_frame``  attribute of
     the form `` t -> [ f_t ]`` for mono sound and
@@ -34,16 +36,20 @@ class AudioClip(Clip):
     Examples
     ---------
     
-    >>> # Plays the note A (a sine wave of frequency 404HZ)
+    >>> # Plays the note A (a sine wave of frequency 440HZ)
     >>> import numpy as np
-    >>> make_frame = lambda t : 2*[ np.sin(404 * 2 * np.pi * t) ]
+    >>> make_frame = lambda t: 2*[ np.sin(440 * 2 * np.pi * t) ]
     >>> clip = AudioClip(make_frame, duration=5)
     >>> clip.preview()
                      
     """
     
-    def __init__(self, make_frame = None, duration=None):
+    def __init__(self, make_frame=None, duration=None, fps=None):
         Clip.__init__(self)
+
+        if fps is not None:
+            self.fps = fps
+
         if make_frame is not None:
             self.make_frame = make_frame
             frame0 = self.get_frame(0)
@@ -57,36 +63,30 @@ class AudioClip(Clip):
     
     @requires_duration
     def iter_chunks(self, chunksize=None, chunk_duration=None, fps=None,
-                    quantize=False, nbytes=2, progress_bar=False):
+                    quantize=False, nbytes=2, logger=None):
         """ Iterator that returns the whole sound array of the clip by chunks
         """
         if fps is None:
-            fps=self.fps
+            fps = self.fps
+        logger = proglog.default_bar_logger(logger)
         if chunk_duration is not None:
             chunksize = int(chunk_duration*fps)
         
         totalsize = int(fps*self.duration)
 
-        if (totalsize % chunksize == 0):
-            nchunks = totalsize // chunksize
-        else:
-            nchunks = totalsize // chunksize + 1
+        nchunks = totalsize // chunksize + 1
 
-        pospos = list(range(0, totalsize,  chunksize))+[totalsize]
-
-        def generator():
-            for i in range(nchunks):
-                tt = (1.0/fps)*np.arange(pospos[i],pospos[i+1])
-                yield self.to_soundarray(tt, nbytes= nbytes, quantize=quantize, fps=fps,
-                                         buffersize=chunksize)
-
-        if progress_bar:
-            return tqdm(generator(), total=nchunks)
-        else:
-            return generator()
+        pospos = np.linspace(0, totalsize, nchunks + 1, endpoint=True, dtype=int)
+        
+        for i in logger.iter_bar(chunk=list(range(nchunks))):
+            size = pospos[i+1] - pospos[i]
+            assert(size <= chunksize)
+            tt = (1.0/fps)*np.arange(pospos[i], pospos[i+1])
+            yield self.to_soundarray(tt, nbytes=nbytes, quantize=quantize,
+                                        fps=fps, buffersize=chunksize)
 
     @requires_duration
-    def to_soundarray(self,tt=None, fps=None, quantize=False, nbytes=2, buffersize=50000):
+    def to_soundarray(self, tt=None, fps=None, quantize=False, nbytes=2, buffersize=50000):
         """
         Transforms the sound into an array that can be played by pygame
         or written in a wav file. See ``AudioClip.preview``.
@@ -106,12 +106,12 @@ class AudioClip(Clip):
         if fps is None:
             fps = self.fps
        
-        stacker = np.vstack if self.nchannels==2 else np.hstack 
+        stacker = np.vstack if self.nchannels == 2 else np.hstack
         max_duration = 1.0 * buffersize / fps
-        if (tt is None):
-            if self.duration>max_duration:
-                return stacker(self.iter_chunks(fps=fps, quantize=quantize, nbytes=2,
-                                                 chunksize=buffersize))
+        if tt is None:
+            if self.duration > max_duration:
+                return stacker(self.iter_chunks(fps=fps, quantize=quantize,
+                                                nbytes=2, chunksize=buffersize))
             else:
                 tt = np.arange(0, self.duration, 1.0/fps)
         """
@@ -127,29 +127,25 @@ class AudioClip(Clip):
         snd_array = self.get_frame(tt)
 
         if quantize:
-            snd_array = np.maximum(-0.99, np.minimum(0.99,snd_array))
-            inttype = {1:'int8',2:'int16', 4:'int32'}[nbytes]
-            snd_array= (2**(8*nbytes-1)*snd_array).astype(inttype)
+            snd_array = np.maximum(-0.99, np.minimum(0.99, snd_array))
+            inttype = {1: 'int8', 2: 'int16', 4: 'int32'}[nbytes]
+            snd_array = (2**(8*nbytes-1)*snd_array).astype(inttype)
         
         return snd_array
 
-    def max_volume(self, stereo=False, chunksize=50000, progress_bar=False):
+    def max_volume(self, stereo=False, chunksize=50000, logger=None):
         
         stereo = stereo and (self.nchannels == 2)
 
-        maxi = np.array([0,0]) if stereo else 0
-        for chunk in self.iter_chunks(chunksize=chunksize, progress_bar=progress_bar):
-            maxi =  np.maximum(maxi,abs(chunk).max(axis=0)) if stereo else max(maxi,abs(chunk).max())
+        maxi = np.array([0, 0]) if stereo else 0
+        for chunk in self.iter_chunks(chunksize=chunksize,logger=logger):
+            maxi = np.maximum(maxi, abs(chunk).max(axis=0)) if stereo else max(maxi, abs(chunk).max())
         return maxi
 
-
-
-    
     @requires_duration
-    def write_audiofile(self,filename, fps=44100, nbytes=2,
-                     buffersize=2000, codec=None,
-                     bitrate=None, ffmpeg_params=None,
-                     write_logfile=False, verbose=True):
+    def write_audiofile(self, filename, fps=None, nbytes=2, buffersize=2000,
+                        codec=None, bitrate=None, ffmpeg_params=None,
+                        write_logfile=False, verbose=True, logger='bar'):
         """ Writes an audio file from the AudioClip.
 
 
@@ -160,9 +156,10 @@ class AudioClip(Clip):
           Name of the output file
 
         fps
-          Frames per second
+          Frames per second. If not set, it will try default to self.fps if
+          already set, otherwise it will default to 44100
 
-        nbyte
+        nbytes
           Sample width (set to 2 for 16-bit sound, 4 for 32-bit sound)
 
         codec
@@ -185,10 +182,17 @@ class AudioClip(Clip):
           when writing the file
 
         verbose
-          If True, displays informations
-
+          Boolean indicating whether to print infomation
+          
+        logger
+          Either 'bar' or None or any Proglog logger
 
         """
+        if not fps:
+            if not self.fps:
+                fps = 44100
+            else:
+                fps = self.fps
 
         if codec is None:
             name, ext = os.path.splitext(os.path.basename(filename))
@@ -196,20 +200,22 @@ class AudioClip(Clip):
                 codec = extensions_dict[ext[1:]]['codec'][0]
             except KeyError:
                 raise ValueError("MoviePy couldn't find the codec associated "
-                       "with the filename. Provide the 'codec' parameter in "
-                       "write_videofile.")
+                                 "with the filename. Provide the 'codec' "
+                                 "parameter in write_audiofile.")
 
         return ffmpeg_audiowrite(self, filename, fps, nbytes, buffersize,
-                      codec=codec, bitrate=bitrate, write_logfile=write_logfile,
-                      verbose=verbose, ffmpeg_params=ffmpeg_params)
+                                 codec=codec, bitrate=bitrate,
+                                 write_logfile=write_logfile, verbose=verbose,
+                                 ffmpeg_params=ffmpeg_params,
+                                 logger=logger)
+
 
-###
-#
 # The to_audiofile method is replaced by the more explicit write_audiofile.
 AudioClip.to_audiofile = deprecated_version_of(AudioClip.write_audiofile,
                                                'to_audiofile')
 ###
 
+
 class AudioArrayClip(AudioClip):
     """
     
@@ -234,16 +240,15 @@ class AudioArrayClip(AudioClip):
         self.array = array
         self.fps = fps
         self.duration = 1.0 * len(array) / fps
-        
-        
+
         def make_frame(t):
             """ complicated, but must be able to handle the case where t
             is a list of the form sin(t) """
             
             if isinstance(t, np.ndarray):
                 array_inds = (self.fps*t).astype(int)
-                in_array = (array_inds>0) & (array_inds < len(self.array))
-                result = np.zeros((len(t),2))
+                in_array = (array_inds > 0) & (array_inds < len(self.array))
+                result = np.zeros((len(t), 2))
                 result[in_array] = self.array[array_inds[in_array]]
                 return result
             else:
@@ -288,12 +293,12 @@ class CompositeAudioClip(AudioClip):
             
             played_parts = [c.is_playing(t) for c in self.clips]
             
-            sounds= [c.get_frame(t - c.start)*np.array([part]).T
-                     for c,part in zip(self.clips, played_parts)
-                     if (part is not False) ]
+            sounds = [c.get_frame(t - c.start)*np.array([part]).T
+                      for c, part in zip(self.clips, played_parts)
+                      if (part is not False)]
                      
-            if isinstance(t,np.ndarray):
-                zero = np.zeros((len(t),self.nchannels))
+            if isinstance(t, np.ndarray):
+                zero = np.zeros((len(t), self.nchannels))
                 
             else:
                 zero = np.zeros(self.nchannels)
@@ -304,7 +309,15 @@ class CompositeAudioClip(AudioClip):
 
 
 def concatenate_audioclips(clips):
+    """
+    The clip with the highest FPS will be the FPS of the result clip.
+    """
     durations = [c.duration for c in clips]
-    tt = np.cumsum([0]+durations) # start times, and end time.
-    newclips= [c.set_start(t) for c,t in zip(clips, tt)]
-    return CompositeAudioClip(newclips).set_duration(tt[-1])
+    tt = np.cumsum([0]+durations)  # start times, and end time.
+    newclips = [c.set_start(t) for c, t in zip(clips, tt)]
+
+    result = CompositeAudioClip(newclips).set_duration(tt[-1])
+
+    fpss = [c.fps for c in clips if getattr(c, 'fps', None)]
+    result.fps = max(fpss) if fpss else None
+    return result
