@@ -1,4 +1,5 @@
-from ..AudioClip import concatenate_audio
+from ..AudioClip import concatenate_audioclips
+
 
 def audio_loop(audioclip, nloops=None, duration=None):
     """ Loops over an audio clip.
@@ -20,9 +21,8 @@ def audio_loop(audioclip, nloops=None, duration=None):
     if duration is not None:
 
         nloops = int( duration/ audioclip.duration)+1
-        return concatenate_audio(nloops*[audioclip]).set_duration(duration)
+        return concatenate_audioclips(nloops*[audioclip]).set_duration(duration)
     
     else:
 
-        return concatenate_audio(nloops*[audioclip])
-    
+        return concatenate_audioclips(nloops*[audioclip])
