@@ -1,17 +1,12 @@
+import os
 import subprocess as sp
-import re
+import warnings
 
 import numpy as np
-from moviepy.tools import cvsecs
 
-from moviepy.video.io.ffmpeg_reader import ffmpeg_parse_infos
+from moviepy.compat import DEVNULL, PY3
 from moviepy.config import get_setting
-
-import os
-try:
-    from subprocess import DEVNULL  # py3k
-except ImportError:
-    DEVNULL = open(os.devnull, 'wb')
+from moviepy.video.io.ffmpeg_reader import ffmpeg_parse_infos
 
 
 class FFMPEG_AudioReader:
@@ -69,8 +64,6 @@ class FFMPEG_AudioReader:
         self.initialize()
         self.buffer_around(1)
 
-
-
     def initialize(self, starttime = 0):
         """ Opens the file, creates the pipe. """
 
@@ -114,12 +107,17 @@ class FFMPEG_AudioReader:
 
 
     def read_chunk(self,chunksize):
+        # chunksize is not being autoconverted from float to int
+        chunksize = int(round(chunksize))
         L = self.nchannels*chunksize*self.nbytes
         s = self.proc.stdout.read(L)
         dt = {1: 'int8',2:'int16',4:'int32'}[self.nbytes]
-        result = np.fromstring(s, dtype=dt)
+        if hasattr(np, 'frombuffer'):
+            result = np.frombuffer(s, dtype=dt)
+        else:
+            result = np.fromstring(s, dtype=dt)
         result = (1.0*result / 2**(8*self.nbytes-1)).\
-                                 reshape((len(result)/self.nchannels,
+                                 reshape((int(len(result)/self.nchannels),
                                           self.nchannels))
         #self.proc.stdout.flush()
         self.pos = self.pos+chunksize
@@ -152,7 +150,8 @@ class FFMPEG_AudioReader:
             for std in [ self.proc.stdout,
                          self.proc.stderr]:
                 std.close()
-            del self.proc
+            self.proc.wait()
+            self.proc = None
 
     def get_frame(self, tt):
 
@@ -164,9 +163,14 @@ class FFMPEG_AudioReader:
 
             # elements of t that are actually in the range of the
             # audio file.
-
             in_time = (tt>=0) & (tt < self.duration)
 
+        # Check that the requested time is in the valid range
+            if not in_time.any():
+                raise IOError("Error in file %s, "%(self.filename)+
+                       "Accessing time t=%.02f-%.02f seconds, "%(tt[0], tt[-1])+
+                       "with clip duration=%d seconds, "%self.duration)
+
             # The np.round in the next line is super-important.
             # Removing it results in artifacts in the noise.
             frames = np.round((self.fps*tt)).astype(int)[in_time]
@@ -184,13 +188,22 @@ class FFMPEG_AudioReader:
             try:
                 result = np.zeros((len(tt),self.nchannels))
                 indices = frames - self.buffer_startframe
+                if len(self.buffer) < self.buffersize // 2:
+                    indices = indices - (self.buffersize // 2 - len(self.buffer) + 1)
                 result[in_time] = self.buffer[indices]
                 return result
+
             except IndexError as error:
-                raise IOError("Error in file %s, "%(self.filename)+
+                warnings.warn("Error in file %s, "%(self.filename)+
                        "At time t=%.02f-%.02f seconds, "%(tt[0], tt[-1])+
                        "indices wanted: %d-%d, "%(indices.min(), indices.max())+
-                       "but len(buffer)=%d\n"%(len(self.buffer))+ str(error))
+                       "but len(buffer)=%d\n"%(len(self.buffer))+ str(error),
+                   UserWarning)
+
+                # repeat the last frame instead
+                indices[indices>=len(self.buffer)] = len(self.buffer) -1
+                result[in_time] = self.buffer[indices]
+                return result
 
         else:
 
@@ -237,7 +250,5 @@ class FFMPEG_AudioReader:
 
 
     def __del__(self):
+        # If the garbage collector comes, make sure the subprocess is terminated.
         self.close_proc()
-
-
-
