@@ -1,37 +1,35 @@
 import os
 import subprocess as sp
-try:
-    from subprocess import DEVNULL  # py3k
-except ImportError:
-    DEVNULL = open(os.devnull, 'wb')
-    
+
+from .compat import DEVNULL
+from .config_defaults import FFMPEG_BINARY, IMAGEMAGICK_BINARY
+
 if os.name == 'nt':
-    try:    
+    try:
         import winreg as wr # py3k
-    except:
+    except ImportError:
         import _winreg as wr # py2k
 
-from .config_defaults import (FFMPEG_BINARY, IMAGEMAGICK_BINARY)
 
 def try_cmd(cmd):
-        try:
-            popen_params = { "stdout": sp.PIPE,
-                             "stderr": sp.PIPE,
-                              "stdin": DEVNULL
-                            }
+    try:
+        popen_params = {
+            "stdout": sp.PIPE,
+            "stderr": sp.PIPE,
+            "stdin": DEVNULL
+        }
 
-            
-            # This was added so that no extra unwanted window opens on windows
-            # when the child process is created
-            if os.name == "nt":
-                popen_params["creationflags"] = 0x08000000
+        # This was added so that no extra unwanted window opens on windows
+        # when the child process is created
+        if os.name == "nt":
+            popen_params["creationflags"] = 0x08000000
 
-            proc = sp.Popen(cmd, **popen_params)
-            proc.communicate()
-        except Exception as err:
-            return False, err
-        else:
-            return True, None
+        proc = sp.Popen(cmd, **popen_params)
+        proc.communicate()
+    except Exception as err:
+        return False, err
+    else:
+        return True, None
 
 if FFMPEG_BINARY=='ffmpeg-imageio':
     from imageio.plugins.ffmpeg import get_exe
@@ -48,13 +46,12 @@ elif FFMPEG_BINARY=='auto-detect':
 else:
     success, err = try_cmd([FFMPEG_BINARY])
     if not success:
-        raise IOError(err.message +
-                 "The path specified for the ffmpeg binary might be wrong")
-
-
+        raise IOError(
+            str(err) +
+            " - The path specified for the ffmpeg binary might be wrong")
 
 if IMAGEMAGICK_BINARY=='auto-detect':
-    if os.name == 'nt':    
+    if os.name == 'nt':
         try:
             key = wr.OpenKey(wr.HKEY_LOCAL_MACHINE, 'SOFTWARE\\ImageMagick\\Current')
             IMAGEMAGICK_BINARY = wr.QueryValueEx(key, 'BinPath')[0] + r"\convert.exe"
@@ -66,15 +63,28 @@ if IMAGEMAGICK_BINARY=='auto-detect':
     else:
         IMAGEMAGICK_BINARY = 'unset'
 else:
+    if not os.path.exists(IMAGEMAGICK_BINARY):
+        raise IOError(
+            "ImageMagick binary cannot be found at {}".format(
+                IMAGEMAGICK_BINARY
+            )
+        )
+
+    if not os.path.isfile(IMAGEMAGICK_BINARY):
+        raise IOError(
+            "ImageMagick binary found at {} is not a file".format(
+                IMAGEMAGICK_BINARY
+            )
+        )
+
     success, err = try_cmd([IMAGEMAGICK_BINARY])
     if not success:
-        raise IOError(err.message +
-                 "The path specified for the ImageMagick binary might be wrong")
-
+        raise IOError("%s - The path specified for the ImageMagick binary might "
+                      "be wrong: %s" % (err, IMAGEMAGICK_BINARY))
 
 
 def get_setting(varname):
-    """ Returns the value of a configuration variable. """ 
+    """ Returns the value of a configuration variable. """
     gl = globals()
     if varname not in gl.keys():
         raise ValueError("Unknown setting %s"%varname)
@@ -83,16 +93,19 @@ def get_setting(varname):
     return gl[varname]
 
 
-def change_settings(new_settings={}, file=None):
+def change_settings(new_settings=None, filename=None):
     """ Changes the value of configuration variables."""
+    new_settings = new_settings or {}
     gl = globals()
-    if file is not None:
-        execfile(file)
+    if filename:
+        with open(filename) as in_file:
+            exec(in_file)
         gl.update(locals())
     gl.update(new_settings)
     # Here you can add some code  to check that the new configuration
     # values are valid.
 
+
 if __name__ == "__main__":
     if try_cmd([FFMPEG_BINARY])[0]:
         print( "MoviePy : ffmpeg successfully found." )
@@ -103,5 +116,3 @@ if __name__ == "__main__":
         print( "MoviePy : ImageMagick successfully found." )
     else:
         print( "MoviePy : can't find or access ImageMagick." )
-
-
