@@ -17,8 +17,22 @@ clip.preview().
 # Note that these imports could have been performed in the __init__.py
 # file, but this would make the loading of moviepy slower.
 
-# Clips
+import os
+import sys
 
+# Downloads ffmpeg if it isn't already installed
+import imageio
+# Checks to see if the user has set a place for their own version of ffmpeg
+
+if os.getenv('FFMPEG_BINARY') is None:
+    if sys.version_info < (3, 4):
+        #uses an old version of imageio with ffmpeg.download.
+        imageio.plugins.ffmpeg.download()
+
+# Hide the welcome message from pygame: https://github.com/pygame/pygame/issues/542
+os.environ['PYGAME_HIDE_SUPPORT_PROMPT'] = "1"
+
+# Clips
 from .video.io.VideoFileClip import VideoFileClip
 from .video.io.ImageSequenceClip import ImageSequenceClip
 from .video.io.downloader import download_webfile
@@ -53,6 +67,7 @@ except ImportError:
 for method in [
           "afx.audio_fadein",
           "afx.audio_fadeout",
+          "afx.audio_normalize",
           "afx.volumex",
           "transfx.crossfadein",
           "transfx.crossfadeout",
@@ -69,16 +84,17 @@ for method in [
           "vfx.speedx"
           ]:
 
-    exec("VideoClip.%s = %s"%( method.split('.')[1], method))
+    exec("VideoClip.%s = %s" % (method.split('.')[1], method))
 
 
 for method in ["afx.audio_fadein",
                "afx.audio_fadeout",
                "afx.audio_loop",
+               "afx.audio_normalize",
                "afx.volumex"
               ]:
               
-    exec("AudioClip.%s = %s"%( method.split('.')[1], method))
+    exec("AudioClip.%s = %s" % (method.split('.')[1], method))
 
 
 # adds easy ipython integration
@@ -96,6 +112,7 @@ except ImportError:
     def preview(self, *args, **kwargs):
         """NOT AVAILABLE : clip.preview requires Pygame installed."""
         raise ImportError("clip.preview requires Pygame installed")
+
     def show(self, *args, **kwargs):
         """NOT AVAILABLE : clip.show requires Pygame installed."""
         raise ImportError("clip.show requires Pygame installed")
@@ -111,4 +128,4 @@ except ImportError:
         """ NOT AVAILABLE : clip.preview requires Pygame installed."""
         raise ImportError("clip.preview requires Pygame installed")
 
-AudioClip.preview = preview
\ No newline at end of file
+AudioClip.preview = preview
