@@ -1,21 +1,23 @@
 """
 Misc. useful functions that can be used at many places in the program.
 """
-
+import os
 import subprocess as sp
 import sys
 import warnings
-import re
 
-import os
-try:
-    from subprocess import DEVNULL  # py3k
-except ImportError:
-    DEVNULL = open(os.devnull, 'wb')
+import proglog
+
+from .compat import DEVNULL
+
 
 
 def sys_write_flush(s):
     """ Writes and flushes without delay a text in the console """
+    # Reason for not using `print` is that in some consoles "print" 
+    # commands get delayed, while stdout.flush are instantaneous, 
+    # so this method is better at providing feedback.
+    # See https://github.com/Zulko/moviepy/pull/485
     sys.stdout.write(s)
     sys.stdout.flush()
 
@@ -26,10 +28,13 @@ def verbose_print(verbose, s):
         sys_write_flush(s)
 
 
-def subprocess_call(cmd, verbose=True, errorprint=True):
-    """ Executes the given subprocess command."""
-
-    verbose_print(verbose, "\n[MoviePy] Running:\n>>> "+ " ".join(cmd))
+def subprocess_call(cmd, logger='bar', errorprint=True):
+    """ Executes the given subprocess command.
+    
+    Set logger to None or a custom Proglog logger to avoid printings.
+    """
+    logger = proglog.default_bar_logger(logger)
+    logger(message='Moviepy - Running:\n>>> "+ " ".join(cmd)')
 
     popen_params = {"stdout": DEVNULL,
                     "stderr": sp.PIPE,
@@ -44,10 +49,11 @@ def subprocess_call(cmd, verbose=True, errorprint=True):
     proc.stderr.close()
 
     if proc.returncode:
-        verbose_print(errorprint, "\n[MoviePy] This command returned an error !")
+        if errorprint:
+            logger(message='Moviepy - Command returned an error')
         raise IOError(err.decode('utf8'))
     else:
-        verbose_print(verbose, "\n... command successful.\n")
+        logger(message='Moviepy - Command successful')
 
     del proc
 
@@ -59,39 +65,41 @@ def is_string(obj):
     except NameError:
         return isinstance(obj, str)
 
+
 def cvsecs(time):
-    """ Will convert any time into seconds.
-    Here are the accepted formats:
+    """ Will convert any time into seconds. 
+    
+    If the type of `time` is not valid, 
+    it's returned as is. 
 
-    >>> cvsecs(15.4) -> 15.4 # seconds
-    >>> cvsecs( (1,21.5) ) -> 81.5 # (min,sec)
-    >>> cvsecs( (1,1,2) ) -> 3662 # (hr, min, sec)
-    >>> cvsecs('01:01:33.5') -> 3693.5  #(hr,min,sec)
-    >>> cvsecs('01:01:33.045') -> 3693.045
-    >>> cvsecs('01:01:33,5') #coma works too
+    Here are the accepted formats::
+
+    >>> cvsecs(15.4)   # seconds 
+    15.4 
+    >>> cvsecs((1, 21.5))   # (min,sec) 
+    81.5 
+    >>> cvsecs((1, 1, 2))   # (hr, min, sec)  
+    3662  
+    >>> cvsecs('01:01:33.045') 
+    3693.045
+    >>> cvsecs('01:01:33,5')    # coma works too
+    3693.5
+    >>> cvsecs('1:33,5')    # only minutes and secs
+    99.5
+    >>> cvsecs('33.5')      # only secs
+    33.5
     """
+    factors = (1, 60, 3600)
+    
+    if is_string(time):     
+        time = [float(f.replace(',', '.')) for f in time.split(':')]
 
-    if is_string(time):
-        if (',' not in time) and ('.' not in time):
-            time = time + '.0'
-        expr = r"(\d+):(\d+):(\d+)[,|.](\d+)"
-        finds = re.findall(expr, time)[0]
-        nums = list( map(float, finds) )
-        return ( 3600*int(finds[0])
-                + 60*int(finds[1])
-                + int(finds[2])
-                + nums[3]/(10**len(finds[3])))
-
-    elif isinstance(time, tuple):
-        if len(time)== 3:
-            hr, mn, sec = time
-        elif len(time)== 2:
-            hr, mn, sec = 0, time[0], time[1]
-        return 3600*hr + 60*mn + sec
-
-    else:
+    if not isinstance(time, (tuple, list)):
         return time
 
+    return sum(mult * part for mult, part in zip(factors, reversed(time)))
+
+
 def deprecated_version_of(f, oldname, newname=None):
     """ Indicates that a function is deprecated and has a new name.
 
@@ -137,7 +145,7 @@ def deprecated_version_of(f, oldname, newname=None):
 # Note that 'gif' is complicated to place. From a VideoFileClip point of view,
 # it is a video, but from a HTML5 point of view, it is an image.
 
-extensions_dict = { "mp4":  {'type':'video', 'codec':['libx264','libmpeg4']},
+extensions_dict = { "mp4":  {'type':'video', 'codec':['libx264','libmpeg4', 'aac']},
                     'ogv':  {'type':'video', 'codec':['libtheora']},
                     'webm': {'type':'video', 'codec':['libvpx']},
                     'avi':  {'type':'video'},
@@ -145,15 +153,25 @@ extensions_dict = { "mp4":  {'type':'video', 'codec':['libx264','libmpeg4']},
 
                     'ogg':  {'type':'audio', 'codec':['libvorbis']},
                     'mp3':  {'type':'audio', 'codec':['libmp3lame']},
-                    'wav':  {'type':'audio', 'codec':['pcm_s16le', 'pcm_s32le']},
+                    'wav':  {'type':'audio', 'codec':['pcm_s16le', 'pcm_s24le', 'pcm_s32le']},
                     'm4a':  {'type':'audio', 'codec':['libfdk_aac']}
                   }
 
 for ext in ["jpg", "jpeg", "png", "bmp", "tiff"]:
     extensions_dict[ext] = {'type':'image'}
 
+
 def find_extension(codec):
+    if codec in extensions_dict:
+        # codec is already the extension
+        return codec
+
     for ext,infos in extensions_dict.items():
-        if ('codec' in infos) and codec in infos['codec']:
+        if codec in infos.get('codec', []):
             return ext
-    raise ValueError
\ No newline at end of file
+    raise ValueError(
+        "The audio_codec you chose is unknown by MoviePy. "
+        "You should report this. In the meantime, you can "
+        "specify a temp_audiofile with the right extension "
+        "in write_videofile."
+    )
