@@ -1,6 +1,7 @@
 import numpy as np
-from moviepy.video.VideoClip import VideoClip, ColorClip
+
 from moviepy.audio.AudioClip import CompositeAudioClip
+from moviepy.video.VideoClip import ColorClip, VideoClip
 
 #  CompositeVideoClip
 
@@ -12,6 +13,7 @@ class CompositeVideoClip(VideoClip):
     base class for most compositions.
     
     Parameters
+    ----------
 
     size
       The size (height x width) of the final clip.
@@ -32,9 +34,14 @@ class CompositeVideoClip(VideoClip):
     bg_color
       Color for the unmasked and unfilled regions. Set to None for these
       regions to be transparent (will be slower).
+
+    use_bgclip
+      Set to True if the first clip in the list should be used as the
+      'background' on which all other clips are blitted. That first clip must
+      have the same size as the final clip. If it has no transparency, the final
+      clip will have no mask. 
     
-    If all clips with a fps attribute have the same fps, it becomes the fps of
-    the result.
+    The clip with the highest FPS will be the FPS of the composite clip.
 
     """
 
@@ -43,18 +50,18 @@ class CompositeVideoClip(VideoClip):
 
         if size is None:
             size = clips[0].size
-        
-        if bg_color is None:
-            bg_color = 0.0 if ismask else (0, 0, 0)
 
+        
         if use_bgclip and (clips[0].mask is None):
             transparent = False
         else:
             transparent = (bg_color is None)
         
-        fps_list = list(set([c.fps for c in clips if hasattr(c,'fps')]))
-        if len(fps_list)==1:
-            self.fps= fps_list[0]
+        if bg_color is None:
+            bg_color = 0.0 if ismask else (0, 0, 0)
+
+        fpss = [c.fps for c in clips if getattr(c, 'fps', None)]
+        self.fps = max(fpss) if fpss else None
 
         VideoClip.__init__(self)
         
@@ -66,30 +73,34 @@ class CompositeVideoClip(VideoClip):
         if use_bgclip:
             self.bg = clips[0]
             self.clips = clips[1:]
+            self.created_bg = False
         else:
             self.clips = clips
-            self.bg = ColorClip(size, col=self.bg_color)
+            self.bg = ColorClip(size, color=self.bg_color)
+            self.created_bg = True
 
         
-        
         # compute duration
         ends = [c.end for c in self.clips]
-        if not any([(e is None) for e in ends]):
-            self.duration = max(ends)
-            self.end = max(ends)
+        if None not in ends:
+            duration = max(ends)
+            self.duration = duration
+            self.end = duration
 
         # compute audio
         audioclips = [v.audio for v in self.clips if v.audio is not None]
-        if len(audioclips) > 0:
+        if audioclips:
             self.audio = CompositeAudioClip(audioclips)
 
         # compute mask if necessary
         if transparent:
-            maskclips = [c.mask.set_pos(c.pos) for c in self.clips
-                         if c.mask is not None]
-            if maskclips != []:
-                self.mask = CompositeVideoClip(maskclips,self.size, ismask=True,
-                                               bg_color=0)
+            maskclips = [(c.mask if (c.mask is not None) else
+                          c.add_mask().mask).set_position(c.pos)
+                          .set_end(c.end).set_start(c.start, change_end=False)
+                          for c in self.clips]
+
+            self.mask = CompositeVideoClip(maskclips,self.size, ismask=True,
+                                               bg_color=0.0)
 
         def make_frame(t):
             """ The clips playing at time `t` are blitted over one
@@ -107,6 +118,16 @@ class CompositeVideoClip(VideoClip):
             actually playing at the given time `t`. """
         return [c for c in self.clips if c.is_playing(t)]
 
+    def close(self):
+        if self.created_bg and self.bg:
+            # Only close the background clip if it was locally created.
+            # Otherwise, it remains the job of whoever created it.
+            self.bg.close()
+            self.bg = None
+        if hasattr(self, "audio") and self.audio:
+            self.audio.close()
+            self.audio = None
+
 
 
 def clips_array(array, rows_widths=None, cols_widths=None,
@@ -140,19 +161,16 @@ def clips_array(array, rows_widths=None, cols_widths=None,
     xx = np.cumsum([0]+list(cols_widths)) 
     yy = np.cumsum([0]+list(rows_widths))
     
-    for j,(x,cw) in list(enumerate(zip(xx[:-1],cols_widths))):
-        for i,(y,rw) in list(enumerate(zip(yy[:-1],rows_widths))):
-            clip = array[i,j]
-            w,h = clip.size
+    for j, (x, cw) in enumerate(zip(xx[:-1], cols_widths)):
+        for i, (y, rw) in enumerate(zip(yy[:-1], rows_widths)):
+            clip = array[i, j]
+            w, h = clip.size
             if (w < cw) or (h < rw):
-                clip = (CompositeVideoClip([clip.set_pos('center')],
+                clip = (CompositeVideoClip([clip.set_position('center')],
                                           size = (cw,rw),
                                           bg_color = bg_color).
                                      set_duration(clip.duration))
                 
-            array[i,j] = clip.set_pos((x,y))
+            array[i, j] = clip.set_position((x, y))
                  
-    return CompositeVideoClip(array.flatten(), size = (xx[-1],yy[-1]),
-                              bg_color = bg_color)
-    
-    
+    return CompositeVideoClip(array.flatten(), size=(xx[-1], yy[-1]), bg_color=bg_color)
