@@ -1,19 +1,24 @@
 import numpy as np
 
-from moviepy.tools import deprecated_version_of
-from moviepy.video.VideoClip import VideoClip, ColorClip
-from moviepy.video.compositing.CompositeVideoClip import CompositeVideoClip
 from moviepy.audio.AudioClip import CompositeAudioClip
+from moviepy.tools import deprecated_version_of
+from moviepy.video.compositing.CompositeVideoClip import CompositeVideoClip
+from moviepy.video.compositing.on_color import on_color
+from moviepy.video.VideoClip import ColorClip, VideoClip
+
+try:               # Python 2
+   reduce
+except NameError:  # Python 3
+   from functools import reduce
 
-from moviepy.video.compositing.on_color import on_color 
 
 def concatenate_videoclips(clips, method="chain", transition=None,
                            bg_color=None, ismask=False, padding = 0):
     """ Concatenates several video clips
-    
+
     Returns a video clip made by clip by concatenating several video clips.
     (Concatenated means that they will be played one after another).
-    
+
     There are two methods:
 
     - method="chain": will produce a clip that simply outputs
@@ -33,33 +38,29 @@ def concatenate_videoclips(clips, method="chain", transition=None,
        will be transparent if mask=True, else it will be of the
        color specified by ``bg_color``.
 
-    If all clips with a fps attribute have the same fps, it becomes the fps of
-    the result.
+    The clip with the highest FPS will be the FPS of the result clip.
 
     Parameters
     -----------
-
     clips
       A list of video clips which must all have their ``duration``
       attributes set.
-
     method
       "chain" or "compose": see above.
-
     transition
       A clip that will be played between each two clips of the list.
-    
+
     bg_color
       Only for method='compose'. Color of the background.
       Set to None for a transparent clip
-    
+
     padding
       Only for method='compose'. Duration during two consecutive clips.
       Note that for negative padding, a clip will partly play at the same
       time as the clip it follows (negative padding is cool for clips who fade
       in on one another). A non-null padding automatically sets the method to
       `compose`.
-           
+
     """
 
     if transition is not None:
@@ -67,52 +68,54 @@ def concatenate_videoclips(clips, method="chain", transition=None,
         clips = reduce(lambda x, y: x + y, l) + [clips[-1]]
         transition = None
 
-    
     tt = np.cumsum([0] + [c.duration for c in clips])
 
     sizes = [v.size for v in clips]
 
+    w = max(r[0] for r in sizes)
+    h = max(r[1] for r in sizes)
 
-    w = max([r[0] for r in sizes])
-    h = max([r[1] for r in sizes])
+    tt = np.maximum(0, tt + padding * np.arange(len(tt)))
 
-    tt = np.maximum(0, tt + padding*np.arange(len(tt)))
-    
     if method == "chain":
         def make_frame(t):
             i = max([i for i, e in enumerate(tt) if e <= t])
             return clips[i].get_frame(t - tt[i])
-        
+
+        def get_mask(c):
+            mask = c.mask or ColorClip([1, 1], color=1, ismask=True)
+            if mask.duration is None:
+               mask.duration = c.duration
+            return mask
+
         result = VideoClip(ismask = ismask, make_frame = make_frame)
         if any([c.mask is not None for c in clips]):
-            masks = [c.mask if (c.mask is not None) else
-                     ColorClip([1,1], col=1, ismask=True, duration=c.duration)
-                 #ColorClip(c.size, col=1, ismask=True).set_duration(c.duration)
-                     for c in clips]
-            result.mask = concatenate_videoclips(masks, method="chain", ismask=True)
+            masks = [get_mask(c) for c in clips]
+            result.mask = concatenate_videoclips(masks, method="chain",
+                                                 ismask=True)
             result.clips = clips
-
-
     elif method == "compose":
-        result = CompositeVideoClip( [c.set_start(t).set_pos('center')
+        result = CompositeVideoClip( [c.set_start(t).set_position('center')
                                 for (c, t) in zip(clips, tt)],
                size = (w, h), bg_color=bg_color, ismask=ismask)
+    else:
+        raise Exception("Moviepy Error: The 'method' argument of "
+                        "concatenate_videoclips must be 'chain' or 'compose'")
 
     result.tt = tt
-    
+
     result.start_times = tt[:-1]
     result.start, result.duration, result.end = 0, tt[-1] , tt[-1]
-    
-    audio_t = [(c.audio,t) for c,t in zip(clips,tt) if c.audio is not None]
-    if len(audio_t)>0:
+
+    audio_t = [(c.audio, t) for c, t in zip(clips,tt) if c.audio is not None]
+    if audio_t:
         result.audio = CompositeAudioClip([a.set_start(t)
                                 for a,t in audio_t])
 
-    fps_list = list(set([c.fps for c in clips if hasattr(c,'fps')]))
-    if len(fps_list)==1:
-        result.fps= fps_list[0]
-
+    fpss = [c.fps for c in clips if getattr(c, 'fps', None) is not None]
+    result.fps = max(fpss) if fpss else None
     return result
 
 
-concatenate = deprecated_version_of(concatenate_videoclips, "concatenate_videoclips")
+concatenate = deprecated_version_of(concatenate_videoclips,
+                                    oldname="concatenate")
