@@ -2,7 +2,4 @@
 This module provides classes that make positioning easy
 """
 
-
-class ClipPosition:
-    
-    
+# class ClipPosition:
