@@ -3,17 +3,15 @@ On the long term this will implement several methods to make videos
 out of VideoClips
 """
 
-import subprocess as sp
 import os
+import subprocess as sp
+
 import numpy as np
+from proglog import proglog
 
-try:
-    from subprocess import DEVNULL  # py3k
-except ImportError:
-    DEVNULL = open(os.devnull, 'wb')
-
+from moviepy.compat import DEVNULL, PY3
 from moviepy.config import get_setting
-from moviepy.tools import verbose_print
+
 
 class FFMPEG_VideoWriter:
     """ A class for FFMPEG-based video writing.
@@ -88,7 +86,7 @@ class FFMPEG_VideoWriter:
             '-s', '%dx%d' % (size[0], size[1]),
             '-pix_fmt', 'rgba' if withmask else 'rgb24',
             '-r', '%.02f' % fps,
-            '-i', '-', '-an',
+            '-an', '-i', '-'
         ]
         if audiofile is not None:
             cmd.extend([
@@ -126,22 +124,25 @@ class FFMPEG_VideoWriter:
         # This was added so that no extra unwanted window opens on windows
         # when the child process is created
         if os.name == "nt":
-            popen_params["creationflags"] = 0x08000000
-        
+            popen_params["creationflags"] = 0x08000000  # CREATE_NO_WINDOW
+
         self.proc = sp.Popen(cmd, **popen_params)
 
 
     def write_frame(self, img_array):
         """ Writes one frame in the file."""
         try:
-            self.proc.stdin.write(img_array.tostring())
+            if PY3:
+               self.proc.stdin.write(img_array.tobytes())
+            else:
+               self.proc.stdin.write(img_array.tostring())
         except IOError as err:
-            ffmpeg_error = self.proc.stderr.read()
+            _, ffmpeg_error = self.proc.communicate()
             error = (str(err) + ("\n\nMoviePy error: FFMPEG encountered "
                                  "the following error while writing file %s:"
-                                 "\n\n %s" % (self.filename, ffmpeg_error)))
+                                 "\n\n %s" % (self.filename, str(ffmpeg_error))))
 
-            if "Unknown encoder" in ffmpeg_error:
+            if b"Unknown encoder" in ffmpeg_error:
 
                 error = error+("\n\nThe video export "
                   "failed because FFMPEG didn't find the specified "
@@ -150,7 +151,7 @@ class FFMPEG_VideoWriter:
                   "write_videofile. For instance:\n"
                   "  >>> clip.write_videofile('myvid.webm', codec='libvpx')")%(self.codec)
 
-            elif "incorrect codec parameters ?" in ffmpeg_error:
+            elif b"incorrect codec parameters ?" in ffmpeg_error:
 
                  error = error+("\n\nThe video export "
                   "failed, possibly because the codec specified for "
@@ -164,64 +165,77 @@ class FFMPEG_VideoWriter:
                   "video codec."
                   )%(self.codec, self.ext)
 
-            elif  "encoder setup failed":
+            elif  b"encoder setup failed" in ffmpeg_error:
 
                 error = error+("\n\nThe video export "
                   "failed, possibly because the bitrate you specified "
                   "was too high or too low for the video codec.")
 
+            elif b"Invalid encoder type" in ffmpeg_error:
+
+                error = error + ("\n\nThe video export failed because the codec "
+                  "or file extension you provided is not a video")
+
 
             raise IOError(error)
 
     def close(self):
-        self.proc.stdin.close()
-        if self.proc.stderr is not None:
-            self.proc.stderr.close()
-        self.proc.wait()
+        if self.proc:
+            self.proc.stdin.close()
+            if self.proc.stderr is not None:
+                self.proc.stderr.close()
+            self.proc.wait()
 
-        del self.proc
+        self.proc = None
+
+    # Support the Context Manager protocol, to ensure that resources are cleaned up.
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.close()
 
 def ffmpeg_write_video(clip, filename, fps, codec="libx264", bitrate=None,
                        preset="medium", withmask=False, write_logfile=False,
-                       audiofile=None, verbose=True, threads=None, ffmpeg_params=None):
+                       audiofile=None, verbose=True, threads=None, ffmpeg_params=None,
+                       logger='bar'):
     """ Write the clip to a videofile. See VideoClip.write_videofile for details
     on the parameters.
     """
+    logger = proglog.default_bar_logger(logger)
+
     if write_logfile:
         logfile = open(filename + ".log", 'w+')
     else:
         logfile = None
-
-    verbose_print(verbose, "[MoviePy] Writing video %s\n"%filename)
-    writer = FFMPEG_VideoWriter(filename, clip.size, fps, codec = codec,
+    logger(message='Moviepy - Writing video %s\n' % filename)
+    with FFMPEG_VideoWriter(filename, clip.size, fps, codec = codec,
                                 preset=preset, bitrate=bitrate, logfile=logfile,
                                 audiofile=audiofile, threads=threads,
-                                ffmpeg_params=ffmpeg_params)
+                                ffmpeg_params=ffmpeg_params) as writer:
 
-    nframes = int(clip.duration*fps)
+        nframes = int(clip.duration*fps)
 
-    for t,frame in clip.iter_frames(progress_bar=True, with_times=True,
-                                    fps=fps, dtype="uint8"):
-        if withmask:
-            mask = (255*clip.mask.get_frame(t))
-            if mask.dtype != "uint8":
-                mask = mask.astype("uint8")
-            frame = np.dstack([frame,mask])
-        
-        writer.write_frame(frame)
+        for t,frame in clip.iter_frames(logger=logger, with_times=True,
+                                        fps=fps, dtype="uint8"):
+            if withmask:
+                mask = (255*clip.mask.get_frame(t))
+                if mask.dtype != "uint8":
+                    mask = mask.astype("uint8")
+                frame = np.dstack([frame,mask])
 
-    writer.close()
+            writer.write_frame(frame)
 
     if write_logfile:
         logfile.close()
-
-    verbose_print(verbose, "[MoviePy] Done.\n")
+    logger(message='Moviepy - Done !')
 
 
 def ffmpeg_write_image(filename, image, logfile=False):
     """ Writes an image (HxWx3 or HxWx4 numpy array) to a file, using
         ffmpeg. """
-    
+
     if image.dtype != 'uint8':
           image = image.astype("uint8")
 
