@@ -2,42 +2,68 @@
 the cuts in MoviePy """
 
 from collections import defaultdict
-from moviepy.decorators import use_clip_fps_by_default
+
 import numpy as np
 
+from moviepy.decorators import use_clip_fps_by_default
+
+
 @use_clip_fps_by_default
 def find_video_period(clip,fps=None,tmin=.3):
     """ Finds the period of a video based on frames correlation """
     
-
     frame = lambda t: clip.get_frame(t).flatten()
-    tt = np.arange(tmin,clip.duration,1.0/ fps)[1:]
+    tt = np.arange(tmin, clip.duration, 1.0/ fps)[1:]
     ref = frame(0)
     corrs = [ np.corrcoef(ref, frame(t))[0,1] for t in tt]
     return tt[np.argmax(corrs)]
 
 
 class FramesMatch:
+    """
+    
+    Parameters
+    -----------
 
-    def __init__(self, t1, t2, distance):
-        self.t1, self.t2, self.distance = t1, t2, distance
+    t1
+      Starting time
+
+    t2
+      End time
+
+    d_min
+      Lower bound on the distance between the first and last frames
+
+    d_max
+      Upper bound on the distance between the first and last frames
+
+    """
+
+    def __init__(self, t1, t2, d_min, d_max):
+        self.t1 = t1
+        self.t2 = t2
+        self.d_min = d_min
+        self.d_max = d_max
         self.time_span = t2-t1
 
     def __str__(self):
-        return '(%.04f, %.04f, %.04f)'%(self.t1, self.t2, self.distance)
+
+        return '(%.04f, %.04f, %.04f, %.04f)'%(
+                self.t1, self.t2, self.d_min, self.d_max)
 
     def __repr__(self):
-        return '(%.04f, %.04f, %.04f)'%(self.t1, self.t2, self.distance)
+        return '(%.04f, %.04f, %.04f, %.04f)'%(
+                self.t1, self.t2, self.d_min, self.d_max)
 
     def __iter__(self):
-        return [self.t1, self.t2, self.distance].__iter__()
+        return iter((self.t1, self.t2, self.d_min, self.d_max))
 
 
 class FramesMatches(list):
 
     def __init__(self, lst):
 
-        list.__init__(self, sorted(lst, key=lambda e: e.distance))
+        list.__init__(self, sorted(lst, key=lambda e: e.d_max))
 
     def best(self, n=1, percent=None):
         if percent is not None:
@@ -123,13 +149,13 @@ class FramesMatches(list):
         
         matching_frames = [] # the final result.
         
-        for (t,frame) in clip.iter_frames(with_times=True, progress_bar=True):
+        for (t,frame) in clip.iter_frames(with_times=True, logger='bar'):
             
             flat_frame = 1.0*frame.flatten()
             F_norm_sq = dot_product(flat_frame, flat_frame)
             F_norm = np.sqrt(F_norm_sq)
             
-            for t2 in F.keys():
+            for t2 in list(F.keys()):
                 # forget old frames, add 't' to the others frames
                 # check for early rejections based on differing norms
                 if (t-t2) > max_d:
@@ -166,7 +192,7 @@ class FramesMatches(list):
                         t3t['rejected'] = True
         
             # Store all the good matches (t2,t)
-            matching_frames += [(t1, t, F[t1][t]['min']) for t1 in F
+            matching_frames += [(t1, t, F[t1][t]['min'], F[t1][t]['max']) for t1 in F
                                 if (t1!=t) and not F[t1][t]['rejected']]
                        
         return FramesMatches([FramesMatch(*e) for e in matching_frames])
@@ -192,10 +218,9 @@ class FramesMatches(list):
         if nomatch_thr is None:
             nomatch_thr = match_thr
 
-        
         dict_starts = defaultdict(lambda : [])
-        for (start, end, distance) in self:
-            dict_starts[start].append([end, distance])
+        for (start, end, d_min, d_max) in self:
+            dict_starts[start].append([end, d_min, d_max])
 
         starts_ends = sorted(dict_starts.items(), key = lambda k: k[0])
         
@@ -206,41 +231,37 @@ class FramesMatches(list):
             if start < min_start:
                 continue
 
-            ends = [end for (end, distance) in ends_distances]
-            great_matches = [end for (end,dist) in ends_distances
-                            if dist<match_thr]
+            ends = [end for (end, d_min, d_max) in ends_distances]
+            great_matches = [(end,d_min, d_max)
+                             for (end,d_min, d_max) in ends_distances
+                             if d_max<match_thr]
             
-            great_long_matches = [end for end in great_matches
+            great_long_matches = [(end,d_min, d_max)
+                                  for (end,d_min, d_max) in great_matches
                                   if (end-start)>min_time_span]
             
             
-            if (great_long_matches == []):
+            if not great_long_matches:
                 continue # No GIF can be made starting at this time
             
-            poor_matches = set([end for (end,dist) in ends_distances
-                            if dist>nomatch_thr])
-            short_matches = [end for end in ends
-                             if (end-start)<=0.6]
+            poor_matches = {end for (end,d_min, d_max) in ends_distances if d_min > nomatch_thr}
+            short_matches = {end for end in ends if (end-start) <= 0.6}
             
-            if len( poor_matches.intersection(short_matches) ) == 0 :
+            if not poor_matches.intersection(short_matches):
                 continue
     
-    
-            end = max(great_long_matches)
-            result.append(FramesMatch(start, end, match_thr))
+            end = max(end for (end, d_min, d_max) in great_long_matches)
+            end, d_min, d_max = next(e for e in great_long_matches if e[0]==end)
+            
+            result.append(FramesMatch(start, end, d_min, d_max))
             min_start = start + time_distance
 
-        return FramesMatches( result )
-
+        return FramesMatches(result)
 
 
     def write_gifs(self, clip, gif_dir):
-        """
-
-        """
-
-        for (start, end, _) in self: 
-            name = "%s/%08d_%08d.gif"%(gif_dir, 100*start, 100*end)
+        for (start, end, _, _) in self: 
+            name = "%s/%08d_%08d.gif" % (gif_dir, 100*start, 100*end)
             clip.subclip(start, end).write_gif(name, verbose=False)
 
 
@@ -248,7 +269,7 @@ class FramesMatches(list):
 
 @use_clip_fps_by_default
 def detect_scenes(clip=None, luminosities=None, thr=10,
-                  progress_bar=False, fps=None):
+                  logger='bar', fps=None):
     """ Detects scenes of a clip based on luminosity changes.
     
     Note that for large clip this may take some time
@@ -287,14 +308,11 @@ def detect_scenes(clip=None, luminosities=None, thr=10,
       Must be provided if you provide no clip or a clip without
       fps attribute.
     
-    
-    
-    
-    """
-        
+
+    """       
     if luminosities is None:
         luminosities = [f.sum() for f in clip.iter_frames(
-                             fps=fps, dtype='uint32', progress_bar=1)]
+                             fps=fps, dtype='uint32', logger=logger)]
     
     luminosities = np.array(luminosities, dtype=float)
     if clip is not None:
